@@ -207,6 +207,9 @@ class MSNeuralNetwork(ModeSolver):
         self.num_modes = 1
         self.x = ann.x
         self.y = ann.y
+        self.after_x = 0.5*(self.x[:-1]+self.x[1:])
+        self.after_y = 0.5*(self.y[:-1]+self.y[1:])
+        self.mesh = len(self.x) - 1
 
 
     def solve(self):
@@ -335,6 +338,7 @@ class MSNeuralNetwork(ModeSolver):
         Hx, Hy, neff = self.mode
         m = Mode(self.x, self.y, self.wl, neff, Hx+0j, Hy+0j, None, None, None, None,width=self.width, thickness=self.thickness)
         m.compute_other_fields()
+        m.normalize()
 
         return m
 
