@@ -4,6 +4,8 @@ from simphony.pins import Pin
 from simphony.tools import wl2freq
 from simphony.models import Subcircuit
 from matplotlib import pyplot as plt
+from emepy.monitors import Monitor
+from copy import copy
 
 
 class Layer(object):
@@ -98,6 +100,7 @@ class EME(object):
         self.layers = layers
         self.num_periods = num_periods
         self.s_params = None
+        self.monitors = []
 
     def add_layer(self, layer):
         """The add_layer method will add a Layer object to the EME object. The object will be geometrically added to the very right side of the structure. Using this method after propagate is useless as the solver has already been called.
@@ -111,17 +114,22 @@ class EME(object):
 
         self.layers.append(layer)
 
-    def reset(self):
+    def reset(self, full_reset=True):
         """Clears out the layers and s params so the user can reuse the object in memory on a new geometry
         """
 
-        self.layers = []
-        self.interfaces = []
-        self.wavelength = None
+        if full_reset:
+            self.layers = []
+            self.wavelength = None
+        else:
+            for i in range(len(self.layers)):
+                self.layers[i].clear()
+
         self.s_params = None
         self.interface = None
+        self.monitors = []
 
-    def propagate_period(self):
+    def propagate_period(self, input_array):
         """The propagate_period method should be called once all Layer objects have been added. This method will call the EME solver and produce s-parameters for ONE period of the structure. If num_periods is set to 1 (default), this method is the same as propagate, except for it returns values. 
 
         Returns
@@ -141,6 +149,12 @@ class EME(object):
         mode_set1 = self.layers[0].get_activated_layer()
         self.layers[1].activate_layer()
         current = Current(self.wavelength, self.layers[0].get_activated_layer())
+
+        # Update the monitors
+        self.update_monitors(
+            self.layers[0].length, self.layers[0].length, mode_set1, not_first=False, input_array=input_array
+        )
+
         interface = self.interface(self.layers[0].get_activated_layer(), self.layers[1].get_activated_layer())
         interface.solve()
         self.layers[0].clear()
@@ -157,6 +171,11 @@ class EME(object):
             layer1 = layer1_.get_activated_layer()
             layer2 = layer2_.get_activated_layer()
 
+            # Update the monitors
+            self.update_monitors(
+                self.layers[index].length, self.layers[index].length, layer1, current, input_array=input_array
+            )
+
             interface = self.interface(layer1, layer2)
             interface.solve()
 
@@ -165,23 +184,39 @@ class EME(object):
             current.update_s(self.cascade(Current(self.wavelength, current), interface), interface)
             interface.clear()
 
+        # Gather and return the s params and edge layers
+        mode_set2 = self.layers[-1].get_activated_layer()
+
+        # Update the monitors
+        self.update_monitors(
+            self.layers[-1].length, self.layers[-1].length, mode_set2, current, input_array=input_array
+        )
+
         # Propagate final two layers
         current.update_s(
             self.cascade(Current(self.wavelength, current), self.layers[-1].get_activated_layer()),
             self.layers[-1].get_activated_layer(),
         )
 
-        # Gather and return the s params and edge layers
-        mode_set2 = self.layers[-1].get_activated_layer()
         self.layers[-1].clear()
         self.s_params = current.s_params
+        for m in range(len(self.monitors)):
+            self.monitors[m].normalize()
 
         return (current.s_params, mode_set1, mode_set2)
 
-    def propagate(self):
+    def propagate(self, input_array=None):
         """The propagate method should be called once all Layer objects have been added. This method will call the EME solver and produce s-parameters.
+
+        Parameters
+        ----------
+        input_array : numpy array
+            the array representing the input to the device in s paramter format: [left port mode 1, left port mode 2, ... left port mode n, right port mode 1, right port mode 2, ... right port mode n] (default : [1,0,0,...]) the default represents sending in the fundamental mode of the left port
         """
 
+        if input_array is None:
+            input_array = np.array([1] + [0 for i in range(self.layers[0].num_modes - 1 + self.layers[-1].num_modes)])
+
         # Check for layers
         if not len(self.layers):
             raise Exception("Must place layers before propagating")
@@ -193,7 +228,8 @@ class EME(object):
         self.interface = InterfaceSingleMode if num_modes == 1 else InterfaceMultiMode
 
         # Run the eme for one period and collect s params and edge layers
-        single_period, left, right = self.propagate_period()
+        single_period, left, right = self.propagate_period(input_array)
+        self.interface = InterfaceMultiMode
 
         # Create an interface between the two returns layers
         period_layer = PeriodicLayer(left.modes, right.modes, single_period)
@@ -205,18 +241,144 @@ class EME(object):
         self.layers[0].clear()
         self.layers[1].clear()
 
-        # print(current_layer.s_params)
-
         # Cascade params for each period
-        from copy import copy
+        for l in range(self.num_periods - 1):
 
-        for _ in range(self.num_periods - 1):
+            # Update the monitors
+            self.update_monitors(
+                None,
+                self.get_total_length(),
+                None,
+                curr_s=current_layer,
+                not_first=True,
+                input_array=input_array,
+                periodic=l,
+            )
 
             current_layer.s_params = self.cascade((current_layer), (interface))
             current_layer.s_params = self.cascade((current_layer), (period_layer))
-            # current_layer.s_params = self.cascade(current_layer, period_layer)
 
         self.s_params = current_layer.s_params
+        self.output = np.matmul(current_layer.s_params[0], input_array)
+
+    def get_field(self, mode_set, m, c, curr_s, input_array):
+        # Eigenvalues of all the modes
+        eigenvalues = np.array(
+            [(2 * np.pi) * mode_set.modes[i].neff / (self.wavelength) for i in range(len(mode_set.modes))]
+        ).astype(np.complex128)
+
+        # Phase propagation from the last interfact until the current position
+        phase_prop = np.exp(
+            (self.monitors[m].remaining_lengths[c][0] - self.monitors[m].cur_length[c]) * 1j * eigenvalues
+        )
+
+        # S parameters from mode overlaps and phase prop from previous layers
+        if not (curr_s is None):
+            interface_prop = np.matmul(curr_s.s_parameters()[0], input_array)[-len(phase_prop) :]
+        else:
+            interface_prop = input_array[: len(phase_prop)]
+
+        # Coefficients of each mode
+        mode_coefficients = phase_prop * interface_prop
+
+        # Index doesn't need s param involvement
+        if self.monitors[m].components[c] == "n":
+            field = getattr(mode_set.modes[0], self.monitors[m].components[c])
+        else:
+            # Combine the coefficient defined linear combination of modes to get the field profile
+            field = getattr(mode_set.modes[0], self.monitors[m].components[c]) * mode_coefficients[0]
+            for i in range(1, len(mode_set.modes)):
+                field += getattr(mode_set.modes[i], self.monitors[m].components[c]) * mode_coefficients[i]
+
+        return field
+
+    def update_monitors(
+        self, location_bonus, adder, mode_set, curr_s=None, not_first=True, input_array=None, periodic=None
+    ):
+
+        # Update the monitors
+        for m in range(len(self.monitors)):  # Loop the monitors
+            for c in range(len(self.monitors[m].components)):  # Loop the field components
+
+                if not (periodic is None):
+                    location = (periodic + 2) * self.get_total_length()
+                elif not_first:
+                    location = location_bonus + self.monitors[m].cur_length[c]
+                else:
+                    location = location_bonus
+
+                if self.monitors[m].axes in ["xy", "yx"]:
+
+                    # Check if not correct location
+                    if (
+                        self.monitors[m].cur_length[c]
+                        <= self.monitors[m].lengths[c][0]
+                        <= self.monitors[m].cur_length[c] + adder
+                    ):
+
+                        # Case when repeating layers
+                        if not (periodic is None):
+                            mode_set = self.monitors[m].layers[0]
+
+                        # Get the field
+                        field = self.get_field(mode_set, m, c, curr_s, input_array)
+
+                        self.monitors[m][c, :, :] = field
+
+                    elif periodic is None:
+                        # If periodic and the correct length is
+                        for n in range(self.num_periods):
+                            if (
+                                self.monitors[m].cur_length[c] + n * self.get_total_length()
+                                <= self.monitors[m].lengths[0]
+                                <= self.monitors[m].cur_length[c] + adder + n * self.get_total_length()
+                            ):
+                                self.monitors[m].layers[0] = mode_set
+                else:
+                    while (
+                        len(self.monitors[m].remaining_lengths[c])
+                        and self.monitors[m].remaining_lengths[c][0] < location
+                    ):
+                        dims = self.monitors[m].dimensions
+                        x = dims[1]
+                        y = dims[1]
+                        if len(dims) == 4:
+                            y = dims[2]
+
+                        # Case when repeating layers
+                        if not (periodic is None):
+                            mode_set = self.monitors[m].layers[
+                                round(
+                                    self.monitors[m].remaining_lengths[c][0] - (periodic + 1) * self.get_total_length(),
+                                    14,
+                                )
+                            ]
+
+                        # Get the field
+                        field = self.get_field(mode_set, m, c, curr_s, input_array)
+
+                        # Only care about the area of the grid of concern
+                        if self.monitors[m].axes == "xz":
+                            field = field[:, int(len(field) / 2)]
+                        elif self.monitors[m].axes == "yz":
+                            field = field[int(len(field) / 2), :]
+                        else:
+                            field = field
+
+                        # Save solved field profiles for referencing from repeated layers
+                        if periodic is None:
+                            self.monitors[m].layers[round(self.monitors[m].remaining_lengths[c][0], 14)] = copy(
+                                mode_set
+                            )
+
+                        # Finally, update the monitor
+                        if self.monitors[m].axes in ["xz", "yz"]:
+                            self.monitors[m][c, 0:y, self.monitors[m].cur_prop_index[c]] = field
+                        else:
+                            self.monitors[m][c, 0:x, 0:y, self.monitors[m].cur_prop_index[c]] = field
+
+                # Update the current length
+                self.monitors[m].cur_length[c] += adder
 
     def cascade(self, first, second):
         """Calculates the s_parameters between two layer objects
@@ -261,6 +423,101 @@ class EME(object):
 
         return self.s_params
 
+    def get_total_length(self):
+        return np.sum([layer.length for layer in self.layers])
+
+    def add_monitor(self, axes="xz", mesh_z=200, z_range=None, location=None):
+        """Creates a monitor associated with the eme object BEFORE the simulation is ran
+
+        Parameters
+        ----------
+        axes : string
+            the spacial axes to capture fields in. Options : 'xz' (default), 'xy', 'xz', 'xyz', 'x', 'y', 'z'. Currently only 'xz' is implemented. Note, propagation is always in z. 
+        mesh_z : int
+            number of mesh points in z (for periodic structures, will be z * num_periods) 
+        z_range : tuple
+            tuple or list of the form (start, end) representing the range of the z values to extract
+        location : float
+            z coordinate where to save data for a 'xy' monitor
+        
+        Returns
+        -------
+        Monitor
+            the newly created Monitor object
+        """
+
+        # Establish mesh
+        components = ["Ex", "Ey", "Ez", "Hx", "Hy", "Hz", "n"]
+        x = self.layers[0].mode_solvers.mesh
+        y = self.layers[0].mode_solvers.mesh
+
+        # Ensure the axes is not still under development
+        if axes in ["xz", "zx", "yz", "zy", "xyz", "yxz", "xzy", "yzx", "zxy", "zyx"]:
+
+            # Create lengths
+            l = self.get_total_length()
+            single_lengths = np.linspace(0, l, mesh_z, endpoint=False).tolist()
+            lengths = []
+            for i in range(self.num_periods):
+                lengths += [np.array(j) + i * l for j in single_lengths]
+            lengths = [lengths for i in range(len(components))]
+
+            # Ensure z range is in proper format
+            try:
+                if z_range is None:
+                    start, end = [lengths[0][0], lengths[0][-1]]
+                else:
+                    start, end = z_range
+            except Exception as _:
+                raise Exception(
+                    "z_range should be a tuple or list of the form (start, end) representing the range of the z values to extract where start and end are floats such as (0, 1e-6) for a 1 µm range"
+                )
+
+            # Fix z mesh if changed by z_range
+            difference_start = lambda list_value: abs(list_value - start)
+            difference_end = lambda list_value: abs(list_value - end)
+            s = min(lengths[0], key=difference_start)
+            e = min(lengths[0], key=difference_end)
+            z = np.sum((s <= lengths[0]) * (e >= lengths[0]))
+
+            # Create monitor dimensions
+            c = len(components)
+            dimensions = (c, y, z) if not (axes in ["xyz", "yxz", "xzy", "yzx", "zxy", "zyx"]) else (c, x, y, z)
+
+            # Create grids
+            grid_x = self.layers[0].mode_solvers.after_x
+            grid_y = self.layers[0].mode_solvers.after_y
+            grid_z = np.linspace(s, e, z)
+
+        elif axes in ["xy", "yx"]:
+
+            # No z_range needed
+            z_range = None
+
+            # Create single length
+            lengths = [[location] for _ in range(len(components))]
+
+            # Create monitor dimensions
+            c = len(components)
+            dimensions = (c, x, y)
+
+            # Create grids
+            grid_x = self.layers[0].mode_solvers.x
+            grid_y = self.layers[0].mode_solvers.y
+            grid_z = np.array([location])
+
+        else:
+            raise Exception(
+                "Monitor setup {} has not yet been implemented. Please choose from the following implemented monitor types: ['xz', 'yz', 'xyz']".format(
+                    axes
+                )
+            )
+
+        # Create monitor
+        monitor = Monitor(axes, dimensions, lengths, components, z_range, grid_x, grid_y, grid_z)
+        self.monitors.append(monitor)
+        return monitor
+
     def draw(self):
         """The draw method sketches a rough approximation for the xz geometry of the structure using pyplot where x is the width of the structure and z is the length. Currently, the drawing is very rough. This will change in the future. 
         """
@@ -746,4 +1003,3 @@ class InterfaceMultiMode(Model):
         """
 
         self.s_params = None
-
