@@ -11,7 +11,7 @@ class Mode(object):
     """Object that holds the field profiles and effective index for an eigenmode
     """
 
-    def __init__(self, x, y, wl, neff, Hx, Hy, Hz, Ex, Ey, Ez, n_profile=None, width=None, thickness=None):
+    def __init__(self, x, y, wl, neff, Hx, Hy, Hz, Ex, Ey, Ez, n=None, width=None, thickness=None):
         """Constructor for Mode Object
 
         Parameters
@@ -52,9 +52,24 @@ class Mode(object):
         self.Ex = np.array(Ex)
         self.Ey = np.array(Ey)
         self.Ez = np.array(Ez)
-        self.n_profile = n_profile
+        self.n = n
         self.width = width
         self.thickness = thickness
+        if True in [
+            self.Ex is None,
+            self.Ey is None,
+            self.Ez is None,
+            self.Hx is None,
+            self.Hy is None,
+            self.Hz is None,
+        ]:
+            self.H = np.sqrt(np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2)
+            self.E = np.sqrt(np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2)
+        if self.n is None:
+            eps_func = tools.get_epsfunc(
+                self.width, self.thickness, 2.5e-6, 2.5e-6, tools.Si(self.wl * 1e6), tools.SiO2(self.wl * 1e6)
+            )
+            self.n = eps_func(self.x, self.y)
 
     def plot(self, operation="Real", colorbar=True):
         """Plots the fields in the mode using pyplot. Should call plt.figure() before and plt.show() or plt.savefig() after
@@ -224,7 +239,8 @@ class Mode(object):
         size = int(np.sqrt(cross.shape[0]))
         cross = cross.reshape((size, size))
 
-        return 0.5 * np.trapz(np.trapz(cross, mode1.x), mode1.y)
+        # cross = mode1.Ex * np.conj(mode2.Hy) - mode1.Ey * np.conj(mode2.Hx)
+        return np.trapz(np.trapz(cross, mode1.x), mode1.y)  # /np.trapz(np.trapz(cross, mode1.x), mode1.y) ### HEY
 
     def inner_product(self, mode2):
         """Takes the inner product between self and the provided Mode
@@ -239,8 +255,8 @@ class Mode(object):
         number 
             the inner product between the modes
         """
-        return self._inner_product(self, mode2)
-        # return self._inner_product(self, mode2) * self._inner_product(mode2, self) / self._inner_product(self, self)
+
+        return self._inner_product(self, mode2)  # / self._inner_product(self, self)
 
     def __str__(self):
         return "Mode Object with effective index of " + str(self.neff)
@@ -322,10 +338,10 @@ class Mode(object):
     def normalize(self):
         """Normalizes the Mode to power 1.
         """
-
         self.zero_phase()
         factor = self.inner_product(self)
         self /= np.sqrt(factor)
+        return
 
     def zero_phase(self):
         """Changes the phase such that the z components are all imaginary and the xy components are all real.
@@ -383,16 +399,9 @@ class Mode(object):
 
     def plot_material(self):
         """Plots the index of refraction profile"""
-        n_profile = self.n_profile
-        if n_profile is None:
-            eps_func = tools.get_epsfunc(
-                self.width, self.thickness, 2.5e-6, 2.5e-6, tools.Si(self.wl * 1e6), tools.SiO2(self.wl * 1e6)
-            )
-            n_profile = eps_func(self.x, self.y)
+        n = self.n
 
-        plt.imshow(
-            np.sqrt(np.real(n_profile)).T, extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6]
-        )
+        plt.imshow(np.sqrt(np.real(n)).T, extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6])
         plt.colorbar()
         plt.title("Index of Refraction")
         plt.xlabel("x (µm)")
@@ -405,12 +414,18 @@ class Mode(object):
 
         core_index = tools.Si(self.wl * 1e6)
         cladding_index = tools.SiO2(self.wl * 1e6)
-        if self.n_profile is None:
+        if self.n is None:
             self.epsfunc = tools.get_epsfunc(
-                self.width, self.thickness, 2.5e-6, 2.5e-6, tools.Si(self.wl * 1e6), tools.SiO2(self.wl * 1e6)
+                self.width,
+                self.thickness,
+                2.5e-6,
+                2.5e-6,
+                tools.Si(self.wl * 1e6),
+                tools.SiO2(self.wl * 1e6),
+                compute=True,
             )
         else:
-            self.epsfunc = lambda x, y: self.n_profile[: len(x), : len(y)]
+            self.epsfunc = lambda x, y: self.n[: len(x), : len(y)]
 
         wl = self.wl
         x = np.array(self.x)
@@ -1564,6 +1579,14 @@ class Mode(object):
         self.Ez = Ezs[0]
         self.x = (self.x[1:] + self.x[:-1]) / 2.0
         self.y = (self.y[1:] + self.y[:-1]) / 2.0
+        self.x = self.x - self.x[int(len(self.x) / 2)]
+        self.y = self.y - self.y[int(len(self.y) / 2)]
+        self.H = np.sqrt(np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2)
+        self.E = np.sqrt(np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2)
+        eps_func = tools.get_epsfunc(
+            self.width, self.thickness, 2.5e-6, 2.5e-6, tools.Si(self.wl * 1e6), tools.SiO2(self.wl * 1e6)
+        )
+        self.n = eps_func(self.x, self.y)
 
         # self.Hz = np.zeros((self.Hx.shape[0],self.Hx.shape[1]),dtype="complex")
         # self.Ex = np.zeros((self.Hx.shape[0],self.Hx.shape[1]),dtype="complex")
